import os
import google.generativeai as genai
import readchar
from .config import load_user_config, save_user_config
from .vision import capture_image, upload_to_gemini
import requests
import playsound
import os
import time
from typing import Union

def generate_audio(message: str, voice: str = "Seoyeon"):
    """
    Text to speech using StreamElements API.

    Parameters:
        message (str): The text to convert to speech.
        voice (str): The voice to use for speech synthesis.

    Returns:
        bytes: The audio content in bytes, or None on failure.
    """
    url = f"https://api.streamelements.com/kappa/v2/speech?voice={voice}&text={message}"
    
    headers = {
        'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36'
    }

    try:
        response = requests.get(url, headers=headers)
        if response.status_code == 200:
            return response.content
        return None
    except requests.exceptions.RequestException:
        return None

def speak(message: str, voice: str = "Brian", folder: str = "", extension: str = ".mp3") -> Union[None, str]:
    """
    Generate speech from text and play the audio.

    Args:
        message (str): The text to be spoken.
        voice (str): The selected voice.
        folder (str): The folder to save the file in (default is the current directory).
        extension (str): The file format (default is ".mp3").

    Returns:
        None or an error message.
    """
    try:
        audio_content = generate_audio(message, voice)
        if audio_content is None:
            return "Error generating speech"

        file_path = os.path.join(folder, f"{voice}{extension}")
        with open(file_path, "wb") as file:
            file.write(audio_content)

        playsound.playsound(file_path)

        # Add delay to ensure playsound finishes before deleting
        time.sleep(1)
        os.remove(file_path)

        return None
    except Exception as e:
        return f"Error playing TTS: {str(e)}"

def select_mode():
    """Allow user to select Default or Custom mode using arrow keys."""
    options = ["Default", "Custom"]
    index = 0
    while True:
        print("\nSelect mode:")
        for i, option in enumerate(options):
            print(f"-->{option}" if i == index else f"    {option}")
        key = readchar.readkey()
        if key == readchar.key.UP:
            index = (index - 1) % len(options)
        elif key == readchar.key.DOWN:
            index = (index + 1) % len(options)
        elif key == readchar.key.ENTER:
            return options[index]
        os.system('cls' if os.name == 'nt' else 'clear')

def initialize_ai():
    """Initialize AI assistant settings."""
    name, description, hotword, api_key, mode = load_user_config()
    if not name or not description or not hotword or not api_key or not mode:
        mode = select_mode()
        if mode == "Default":
            name, description, hotword = "User", "Always remember that you are NOVA", "NOVA"
            api_key = input("Enter your Gemini API Key: ").strip()
        else:
            name = input("Your Name: ").strip()
            description = input("A little bit of persona Description for your AI: ").strip()
            hotword = input("Enter your preferred hotword for activating the AI: ").strip()
            api_key = input("Enter your Gemini API Key: ").strip()
        save_user_config(name, description, hotword, api_key, mode)
    
    genai.configure(api_key=api_key)
    
    system_prompt = f"""
    You are {hotword.upper()}, a customized AI assistant built on N.O.V.A (Next-Gen Operational Virtual Assistant), an advanced AI designed by Suraj Sharma.
    
    🌟 **Core Functionalities**:
    - Real-Time Vision & Object Recognition
    - OCR & Text Reading
    - Speech-to-Text & Voice Interaction
    - TTS (Text-to-Speech)
    - Obstacle Detection for Accessibility
    - Smart Glasses Assistant

    🎯 **Objective**: Be an intelligent, reliable, and futuristic AI companion for **{name}**.

    User-defined persona: **{description}**
    """
    
    return genai.GenerativeModel(
        model_name="gemini-2.0-flash",
        generation_config={
            "temperature": 1.5,
            "top_p": 0.9,
            "top_k": 50,
            "max_output_tokens": 1024,
            "response_mime_type": "text/plain",
        },
        system_instruction=system_prompt,
    )

def process_input(user_input, chat_session, hotword):
    """Process user input and generate AI response."""
    if "VISION" in user_input.upper():
        print("Capturing image... 📷")
        image_path = capture_image()
        if image_path:
            file = upload_to_gemini(image_path)
            query = input("Describe what you want to analyze in this image: ")
            response = chat_session.send_message([file, query])
            return response.text
        return "Failed to capture image. Try again."
    
    if hotword.upper() in user_input.upper():
        response = chat_session.send_message(user_input)
        return response.text
    
    return f"I'm listening only when you call '{hotword}' or use 'Vision'. Please try again."

def main():
    """Main function to run the assistant."""
    model = initialize_ai()
    chat_session = model.start_chat()
    name, _, hotword, _, _ = load_user_config()

    while True:
        user_input = input(f"{name}: ")
        if user_input.lower() in ["exit", "quit"]:
            print("Exiting...")
            break
        response = process_input(user_input, chat_session, hotword)
        print(f"{hotword.upper()}: {response}")
        speak(response)
